//
//  pg_bridge.h
//  pgsql_test
//
//  Created by Marco Bambini on 10/02/15.
//  Copyright (c) 2015 CreoLabs. All rights reserved.
//

#ifndef __POSTGRESQL_DYNAMIC_BRIDGE__
#define __POSTGRESQL_DYNAMIC_BRIDGE__

#include "pg_type.h"
#include "postgres_ext.h"

// opaque types
typedef struct PGresult			PGresult;
typedef struct PGconn			PGconn;

// from libpq-fe.h
typedef enum {
	PGRES_EMPTY_QUERY = 0,		/* empty query string was executed */
	PGRES_COMMAND_OK,			/* a query command that doesn't return
								 * anything was executed properly by the
								 * backend */
	PGRES_TUPLES_OK,			/* a query command that returns tuples was
								 * executed properly by the backend, PGresult
								 * contains the result tuples */
	PGRES_COPY_OUT,				/* Copy Out data transfer in progress */
	PGRES_COPY_IN,				/* Copy In data transfer in progress */
	PGRES_BAD_RESPONSE,			/* an unexpected response was recv'd from the
								 * backend */
	PGRES_NONFATAL_ERROR,		/* notice or warning message */
	PGRES_FATAL_ERROR,			/* query failed */
	PGRES_COPY_BOTH,			/* Copy In/Out data transfer in progress */
	PGRES_SINGLE_TUPLE			/* single tuple from larger resultset */
} ExecStatusType;

typedef enum {
	CONNECTION_OK,
	CONNECTION_BAD,
	/* Non-blocking mode only below here */
	
	/*
	 * The existence of these should never be relied upon - they should only
	 * be used for user feedback or similar purposes.
	 */
	CONNECTION_STARTED,			/* Waiting for connection to be made.  */
	CONNECTION_MADE,			/* Connection OK; waiting to send.     */
	CONNECTION_AWAITING_RESPONSE,		/* Waiting for a response from the
										 * postmaster.        */
	CONNECTION_AUTH_OK,			/* Received authentication; waiting for
								 * backend startup. */
	CONNECTION_SETENV,			/* Negotiating environment. */
	CONNECTION_SSL_STARTUP,		/* Negotiating SSL. */
	CONNECTION_NEEDED			/* Internal state: connect() needed */
} ConnStatusType;


#define PQoidValue		(* (Oid (*)(const PGresult *)) pg_func[0])
#define PQexec			(* (PGresult* (*)(PGconn *conn, const char *query)) pg_func[1])
#define PQconnectdb		(* (PGconn* (*)(const char *conninfo)) pg_func[2])
#define PQclear			(* (void (*)(PGresult *res)) pg_func[3])
#define PQstatus		(* (ConnStatusType (*)(const PGconn *conn)) pg_func[4])
#define PQgetResult		(* (PGresult* (*)(PGconn *conn)) pg_func[5])
#define PQfinish		(* (void (*)(PGconn *conn)) pg_func[6])
#define PQerrorMessage	(* (char* (*)(const PGconn *conn)) pg_func[7])
#define PQfname			(* (char* (*)(const PGresult *res, int field_num)) pg_func[8])
#define PQntuples		(* (int	(*)(const PGresult *res)) pg_func[9])
#define PQresultStatus	(* (ExecStatusType (*)(const PGresult *res)) pg_func[10])
#define PQgetvalue		(* (char* (*)(const PGresult *res, int tup_num, int field_num)) pg_func[11])
#define PQsendQuery		(* (int	(*)(PGconn *conn, const char *query)) pg_func[12])
#define PQcmdTuples		(* (char* (*)(PGresult *res)) pg_func[13])
#define PQgetlength		(* (int	(*)(const PGresult *res, int tup_num, int field_num)) pg_func[14])
#define PQftype			(* (Oid	(*)(const PGresult *res, int field_num)) pg_func[15])
#define PQnfields		(* (int	(*)(const PGresult *res)) pg_func[16])
#define PQgetisnull		(* (int	(*)(const PGresult *res, int tup_num, int field_num)) pg_func[17])

#endif
